@@ -31,18 +31,19 @@
 from __future__ import annotations
 
 import copy
+import functools
 import json
 import os
 import socket
 from types import SimpleNamespace
-from typing import Callable, Generator, Iterator
+from typing import Any, Callable, Dict, Generator, Iterator
 
 import pytest
 from keycloak import KeycloakAdmin, KeycloakOpenID
 from playwright.sync_api import Browser, BrowserContext, Page, expect
 from utils import (
-    get_portal_tile, keycloak_login, keycloak_password_change, legacy_auth_config_create, legacy_auth_config_remove,
-    run_command,
+    get_portal_tile, grant_oidc_privileges, keycloak_login, keycloak_password_change, legacy_auth_config_create,
+    legacy_auth_config_remove, run_command,
 )
 
 from univention.appcenter.actions import get_action
@@ -114,6 +115,15 @@ def keycloak_app_version() -> str:
     return version[0]
 
 
+@pytest.fixture()
+def tracing_page(page, request: pytest.FixtureRequest, ucr_proper):
+    page.context.tracing.start(screenshots=True, snapshots=True)
+
+    yield page
+
+    fixtures.teardown_umc_browser_test(request, ucr_proper, page, page.context, page.context.browser)
+
+
 @pytest.fixture()
 def change_app_setting():
     data = {'app': None, 'configure': None, 'changes': {}}
@@ -192,6 +202,7 @@ def portal_config(ucr_proper: ConfigRegistry) -> SimpleNamespace:
         'title': 'Univention Portal',
         'sso_login_tile': 'Login (Single sign-on)',
         'sso_login_tile_de': 'Anmelden (Single Sign-on)',
+        'sso_oidc_login_tile': 'OIDC Login',
         'tile_name_class': 'portal-tile__name',
         'category_title_class': 'portal-category__title',
         'categories_id': 'portalCategories',
@@ -272,29 +283,37 @@ def browser_type_launch_args(browser_type_launch_args):
     }
 
 
-@pytest.fixture()
-def portal_login_via_keycloak(page: Page, portal_config: SimpleNamespace, keycloak_config: SimpleNamespace):
-    def _func(
-        username: str,
-        password: str,
-        fails_with: str | None = None,
-        new_password: str | None = None,
-        new_password_confirm: str | None = None,
-        verify_login: bool | None = True,
-        url: str | None = portal_config.url,
-        no_login: bool = False,
-    ):
+def __portal_login_func(
+    portal_config: SimpleNamespace,
+    keycloak_config: SimpleNamespace,
+    page: Page,
+    username: str,
+    password: str,
+    fails_with: str | None = None,
+    new_password: str | None = None,
+    new_password_confirm: str | None = None,
+    verify_login: bool | None = True,
+    url: str | None = None,
+    no_login: bool = False,
+    protocol: str | None = 'saml',
+):
+    try:
+        url = url or portal_config.url
         page.goto(url)
         expect(page).to_have_title(portal_config.title)
         lang = page.evaluate('() => window.navigator.userLanguage || window.navigator.language')
         sso_login_tile = portal_config.sso_login_tile if lang == 'en-US' else portal_config.sso_login_tile_de
+        if protocol == 'oidc':
+            sso_login_tile = portal_config.sso_oidc_login_tile
         get_portal_tile(page, sso_login_tile, portal_config).click()
         # login
         keycloak_login(page, keycloak_config, username, password, fails_with=fails_with if not new_password else None, no_login=no_login)
         # check password change
         if new_password:
             new_password_confirm = new_password_confirm or new_password
-            keycloak_password_change(page, keycloak_config, password, new_password, new_password_confirm, fails_with=fails_with)
+            keycloak_password_change(page, keycloak_config, username, password, new_password, new_password_confirm, fails_with=fails_with)
+        if protocol == 'oidc':
+            grant_oidc_privileges(page)
         if fails_with or no_login:
             return page
         # check that we are logged in
@@ -302,12 +321,25 @@ def portal_login_via_keycloak(page: Page, portal_config: SimpleNamespace, keyclo
             header_menu = page.locator(f'#{portal_config.header_menu_id}')
             expect(header_menu, 'header menu not visible').to_be_visible()
         return page
-
-    return _func
+    except Exception:
+        print(page.content())
+        raise
 
 
 @pytest.fixture()
-def keycloak_adm_login(page: Page, keycloak_config: SimpleNamespace):
+def portal_login_via_keycloak(tracing_page: Page, portal_config: SimpleNamespace, keycloak_config: SimpleNamespace):
+    return functools.partial(__portal_login_func, portal_config, keycloak_config, tracing_page)
+
+
+@pytest.fixture()
+def portal_login_via_keycloak_custom_page(portal_config: SimpleNamespace, keycloak_config: SimpleNamespace):
+    return functools.partial(__portal_login_func, portal_config, keycloak_config)
+
+
+@pytest.fixture()
+def keycloak_adm_login(tracing_page: Page, keycloak_config: SimpleNamespace):
+    page = tracing_page
+
     def _func(
         username: str,
         password: str,
@@ -315,13 +347,19 @@ def keycloak_adm_login(page: Page, keycloak_config: SimpleNamespace):
         url: str | None = keycloak_config.url,
         no_login: bool = False,
     ):
-        page.goto(url)
-        expect(page).to_have_title('Univention Corporate Server Single-Sign On')
-        keycloak_login(page, keycloak_config, username, password, fails_with=fails_with, no_login=no_login)
-        # check that we are logged in
-        if not fails_with or not no_login:
-            return page
-        expect(page).to_have_title('Keycloak Administration UI')
+        try:
+            page.goto(url)
+            page.wait_for_load_state('networkidle', timeout=5 * 1000)
+            print(page.content())
+            expect(page).to_have_title('Univention Corporate Server Single-Sign On', timeout=30 * 1000)
+            keycloak_login(page, keycloak_config, username, password, fails_with=fails_with, no_login=no_login)
+            # check that we are logged in
+            if fails_with or no_login:
+                return page
+            expect(page).to_have_title('Keycloak Administration Console')
+        except Exception:
+            print(page.content())
+            raise
         return page
 
     return _func
@@ -460,10 +498,9 @@ def legacy_authorization_setup_oidc(
         keycloak_administrator_connection.delete_client(client_id)
 
 
-@pytest.fixture(params=['frontchannel', 'backchannel'])
-def oidc_client_frontchannel(ucr, request):
-    if request.param == 'frontchannel':
-        pytest.skip("frontchannel logout doesn't currently does not work for external OPs. Skipp all frontchannel logout tests for now.")
+@pytest.fixture()
+def modify_keycloak_clients(request):
+    mod_func: Callable[[Dict[Any, Any]]] = request.param
     modified_clients = []
     oidc_clients = json.loads(run_command(['univention-keycloak', 'oidc/rp', 'get', '--json', '--all']))
 
@@ -472,20 +509,16 @@ def oidc_client_frontchannel(ucr, request):
             continue
         modified_clients.append(client)
         oidc_client_updated = copy.deepcopy(client)
-        oidc_client_updated['frontchannelLogout'] = request.param == 'frontchannel'
+        mod_func(oidc_client_updated)
         oidc_client_updated = json.dumps(oidc_client_updated)
         run_command(['univention-keycloak', 'oidc/rp', 'update', client['clientId'], oidc_client_updated])
-
     yield
-
     for client in modified_clients:
         run_command(['univention-keycloak', 'oidc/rp', 'update', client['clientId'], json.dumps(client)])
 
 
 @pytest.fixture()
 def oidc_client_logout_meachanism(request):
-    if request.param == 'frontchannel':
-        pytest.skip("frontchannel logout doesn't currently does not work for external OPs. Skipp all frontchannel logout tests for now.")
     modified_clients = []
     oidc_clients = json.loads(run_command(['univention-keycloak', 'oidc/rp', 'get', '--json', '--all']))
 
@@ -505,7 +538,7 @@ def oidc_client_logout_meachanism(request):
 
 
 @pytest.fixture()
-def multi_tab_context(browser: Browser, request: pytest.FixtureRequest, ucr) -> Generator[BrowserContext, None, None]:
+def multi_tab_context(browser: Browser, request: pytest.FixtureRequest, ucr_proper) -> Generator[BrowserContext, None, None]:
     context = browser.new_context(ignore_https_errors=True)
     context.set_default_timeout(30 * 1000)
     expect.set_options(timeout=30 * 1000)
@@ -514,4 +547,4 @@ def multi_tab_context(browser: Browser, request: pytest.FixtureRequest, ucr) ->
 
     yield context
 
-    fixtures.teardown_umc_browser_test(request, ucr, context.pages, context, browser)
+    fixtures.teardown_umc_browser_test(request, ucr_proper, context.pages, context, browser)
